-- MOST CODE IS TAKEN FROM QUESTHELPER'S TEXT VIEWER
local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon("HideBlizzard")
assert(HideBlizzard, "Viewer::HideBlizzard does not exist")

local viewer, type, frammis = nil, type, {}

function HideBlizzard:ShowText(text, title, width, border, divide)
	local border = border or 8
	local divide = divide or 4

	if (type(text) == "string") then text = {text} end

	if not frammis[border] then frammis[border] = {} end
	viewer = frammis[border][divide]
	local suffix = string.format("_%d_%d", border, divide)

	if not viewer then
		viewer = CreateFrame("Frame", "HideBlizzard_Viewer", nil)
		viewer:SetFrameStrata("FULLSCREEN_DIALOG")
		viewer:SetPoint("CENTER", UIParent, "TOP", 0, -150)
		viewer:EnableMouse(true)
		viewer:SetMovable(true)
		viewer:SetClampedToScreen(true)
		viewer:RegisterForDrag("LeftButton")
		viewer:SetScript("OnDragStart", function(self) self:StartMoving() end)
		viewer:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
		table.insert(UISpecialFrames, viewer:GetName())
		viewer.title = viewer:CreateFontString()
		viewer.title:SetFont("Fonts\\FRIZQT__.TTF", 14)
		viewer.title:SetPoint("TOPLEFT", viewer, border, -border)
		viewer.title:SetPoint("RIGHT", viewer, -border, 0)
		viewer:SetBackdrop({
		 bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		 edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		 edgeSize = 16,
		 tile = true,
		 tileSize = 16,
		 insets = { left = 4, right = 4, top = 4, bottom = 4 }
		 })
		viewer:SetBackdropColor(0, 0, 0, 1)
		viewer:SetBackdropBorderColor(1, 1, 1, 1)

		viewer.scrollframe = CreateFrame("ScrollFrame", "HideBlizzard_Viewer_ScrollFrame"..suffix, viewer, "UIPanelScrollFrameTemplate")

		viewer.scrollframe:SetPoint("LEFT", viewer, "LEFT", border, 0)
		viewer.scrollframe:SetPoint("TOP", viewer.title, "BOTTOM", 0, -divide)

		viewer.scrollbar = _G["HideBlizzard_Viewer_ScrollFrame".. suffix .."ScrollBar"]
		viewer.scrollbar:SetBackdrop({
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			edgeSize = 12,
			tileSize = 16,
			insets = { left = 0, right = 0, top = 5, bottom = 5 }
		})

		viewer.closebutton = CreateFrame("Button", "HideBlizzard_Viewer_CloseButton".. suffix, viewer, "UIPanelCloseButton")
		viewer.closebutton:SetPoint("TOPRIGHT", viewer)
		viewer.closebutton:SetScript("OnClick", function() viewer:Hide() end)

		viewer.frame = CreateFrame("Frame", "HideBlizzard_Viewer_Frame".. suffix, viewer.scrollframe)
		viewer.scrollframe:SetScrollChild(viewer.frame)

		viewer.text = {}
	end

	local maxw = 0
	for i = 1, #text do
		if not viewer.text[i] then
			viewer.text[i] = viewer.frame:CreateFontString()
			viewer.text[i]:SetFont("Fonts\\FRIZQT__.TTF", 12.5)
			viewer.text[i]:SetJustifyH("LEFT")
			if i > 1 then
				viewer.text[i]:SetPoint("TOPLEFT", viewer.text[i - 1], "BOTTOMLEFT")
			else
				viewer.text[i]:SetPoint("TOPLEFT", viewer.frame)
			end
		end
		viewer.text[i]:Show()
		viewer.text[i]:SetText(text[i] or "No text")

		maxw = math.max(maxw, viewer.text[i]:GetStringWidth())
	end

	for i = #text + 1, #viewer.text do
		viewer.text[i]:Hide()
	end

	viewer:Show()
	viewer.title:SetText(title or "HideBlizzard")
	viewer.scrollframe:SetVerticalScroll(0)

	local w = width or math.min(600, math.max(100, maxw))
	for i = 1, #viewer.text do
		viewer.text[i]:SetWidth(w)
	end
	viewer:SetWidth(w+border * 2)
	viewer.scrollframe:SetWidth(w)
	viewer.frame:SetWidth(w)

	local toth = 0
	for i = 1, #text do
		toth = toth + viewer.text[i]:GetHeight()
	end
	local h = math.max(10, toth)
	local title_h = viewer.title:GetHeight()

	if h > 400 then
		viewer.frame:SetHeight(400)
		viewer.scrollframe:SetHeight(400)
		viewer:SetHeight(420+title_h+border * 2+divide)
		viewer:SetWidth(w+border * 2 + 22)
		viewer.scrollbar:Show()
	else
		viewer.frame:SetHeight(h)
		viewer.scrollframe:SetHeight(h)
		viewer:SetHeight(h+border * 2 + divide+title_h)
		viewer.scrollbar:Hide()
		viewer.closebutton:SetButtonState("PUSHED")
		viewer.closebutton:SetButtonState("NORMAL")
	end
	frammis[border][divide] = viewer
end